# coding: utf-8
"""
Unit tests for pypvwatts.

"""
from .pypvwatts import PVWatts, PVWattsResult
from .pvwattserror import PVWattsValidationError

import unittest
import json

SAMPLE_RESPONSE = """
{
   "inputs":{
      "lat":"40",
      "lon":"-105",
      "system_capacity":"4",
      "azimuth":"180",
      "tilt":"40",
      "array_type":"1",
      "module_type":"1",
      "losses":"10"
   },
   "errors":[

   ],
   "warnings":[

   ],
   "version":"1.0.1",
   "ssc_info":{
      "version":34,
      "build":"Unix 64 bit GNU/C++ Aug 18 2014 13:38:36"
   },
   "station_info":{
      "lat":40.016666412353516,
      "lon":-105.25,
      "elev":1634.0,
      "tz":-7.0,
      "location":"94018",
      "city":"BOULDER",
      "state":"CO",
      "solar_resource_file":"94018.tm2",
      "distance":21235
   },
   "outputs":{
      "ac_monthly":[
         474.3351745605469,
         465.9206237792969,
         628.4765625,
         602.564208984375,
         611.0515747070312,
         591.2024536132812,
         596.1395874023438,
         610.1753540039062,
         598.2145385742188,
         574.7982177734375,
         471.78070068359375,
         458.9857177734375
      ],
      "poa_monthly":[
         136.04103088378906,
         136.04443359375,
         185.7895965576172,
         181.16891479492188,
         185.77963256835938,
         182.52105712890625,
         187.89971923828125,
         193.35572814941406,
         187.40081787109375,
         175.5979461669922,
         137.59872436523438,
         131.25526428222656
      ],
      "solrad_monthly":[
         4.388420581817627,
         4.858729839324951,
         5.993212699890137,
         6.038963794708252,
         5.992891311645508,
         6.084035396575928,
         6.061281204223633,
         6.237281322479248,
         6.246694087982178,
         5.664449691772461,
         4.5866241455078125,
         4.2340407371521
      ],
      "dc_monthly":[
         495.09564208984375,
         487.5823669433594,
         657.702880859375,
         629.8565063476562,
         638.9706420898438,
         618.6126708984375,
         623.68994140625,
         637.5205688476562,
         624.635009765625,
         599.9056396484375,
         492.5662841796875,
         479.1076354980469
      ],
      "ac_annual":6683.64501953125,
      "solrad_annual":5.5322184562683105
   }
}
"""


class Test(unittest.TestCase):
    """
    Unit tests for PVWatts.

    """
    def test_pvwatts_results(self):
        """Test PVWattsResult attrib handling"""
        result = PVWattsResult(json.loads(SAMPLE_RESPONSE))
        self.assertEqual(result.solrad_annual, 5.5322184562683105)
        self.assertEqual(result.station_info['city'], 'BOULDER')

    def test_pypvwatts_validation(self):
        """Test pypvwatts validations"""
        self.assertRaises(PVWattsValidationError, PVWatts.request,
                          system_capacity='a', module_type=1, array_type=1,
                          azimuth=190, tilt=30,
                          dataset='tmy2', losses=0.13,
                          lat=40, lon=-105)
        self.assertRaises(PVWattsValidationError, PVWatts.request,
                          system_capacity=4, module_type='1', array_type=1,
                          azimuth=190, tilt=30,
                          dataset='tmy2', losses=0.13,
                          lat=40, lon=-105)
        self.assertRaises(PVWattsValidationError, PVWatts.request,
                          system_capacity=4, module_type=1, array_type='1',
                          azimuth=190, tilt=30,
                          dataset='tmy2', losses=0.13,
                          lat=40, lon=-105)
        self.assertRaises(PVWattsValidationError, PVWatts.request,
                          system_capacity=4, module_type=1, array_type=1,
                          azimuth=-190, tilt=30,
                          dataset='1', losses=0.13,
                          lat=40, lon=-105)
        self.assertRaises(PVWattsValidationError, PVWatts.request,
                          system_capacity=4, module_type=1, array_type=1,
                          azimuth=190, tilt=100,
                          dataset='1', losses=0.13,
                          lat=40, lon=-105)
        self.assertRaises(PVWattsValidationError, PVWatts.request,
                          system_capacity=4, module_type=1, array_type=1,
                          azimuth=190, tilt=30,
                          dataset='1', losses=0.13,
                          lat=40, lon=-105)
        self.assertRaises(PVWattsValidationError, PVWatts.request,
                          system_capacity=4, module_type=1, array_type=1,
                          azimuth=190, tilt=30,
                          dataset='tmy2', losses=-400,
                          lat=40, lon=-105)
        self.assertRaises(PVWattsValidationError, PVWatts.request,
                          system_capacity=4, module_type=1, array_type=1,
                          azimuth=190, tilt=30,
                          dataset='tmy2', losses=0.13,
                          lat=-100, lon=-105)
        self.assertRaises(PVWattsValidationError, PVWatts.request,
                          system_capacity=4, module_type=1, array_type=1,
                          azimuth=190, tilt=30,
                          dataset='tmy2', losses=0.13,
                          lat=40, lon=400)
        self.assertRaises(PVWattsValidationError, PVWatts.request,
                          system_capacity=4, module_type=1, array_type=1,
                          azimuth=190, tilt=30,
                          dataset='tmy2', losses=0.13,
                          lat=40, lon=-105, timeframe='notvalid')
        self.assertRaises(PVWattsValidationError, PVWatts.request,
                          system_capacity=4, module_type=1, array_type=1,
                          azimuth=190, tilt='a',
                          dataset='tmy2', losses=0.13,
                          lat=40, lon=-105)
        self.assertRaises(PVWattsValidationError, PVWatts.request,
                          system_capacity=4, module_type=1, array_type=1,
                          azimuth=190, tilt=30,
                          dataset='tmy2', losses=0.13,
                          lat=40, lon=-105, dc_ac_ratio=-10)
        self.assertRaises(PVWattsValidationError, PVWatts.request,
                          system_capacity=4, module_type=1, array_type=1,
                          azimuth=190, tilt=30,
                          dataset='tmy2', losses=0.13,
                          lat=40, lon=-105, gcr=10)
        self.assertRaises(PVWattsValidationError, PVWatts.request,
                          system_capacity=4, module_type=1, array_type=1,
                          azimuth=190, tilt=30,
                          dataset='tmy2', losses=0.13,
                          lat=40, lon=-105, inv_eff=0)

    def test_pypvwatts(self):
        """Test pypvwatts"""
        PVWatts.api_key = 'DEMO_KEY'
        results = PVWatts.request(
            system_capacity=4, module_type=1, array_type=1,
            azimuth=190, tilt=30, dataset='tmy2',
            losses=0.13, lat=40, lon=-105)
        self.assert_results(results)

    def test_pypvwatts_instance(self):
        """Test pypvwatts instance based searches"""
        p = PVWatts(api_key='DEMO_KEY')
        results = p.request(
            system_capacity=4, module_type=1, array_type=1,
            azimuth=190, tilt=30, dataset='tmy2',
            losses=0.13, lat=40, lon=-105)
        self.assert_results(results)

    def assert_results(self, results):
        self.assertEqual(results.ac_annual, 7263.046875)
        self.assertEqual(results.solrad_annual, 5.446694850921631)
        self.assertEqual(results.station_info['city'], 'BOULDER')
        self.assertIn(501.930908203125, results.dc_monthly)
        self.assertIn(183.11537170410156, results.poa_monthly)
        self.assertIn(6.103845596313477, results.solrad_monthly)


if __name__ == "__main__":
    unittest.main()



# coding: utf-8
"""
Python wrapper for NREL PVWatt version 5.
"""
from .pvwattsresult import PVWattsResult
from .pvwattserror import PVWattsError, PVWattsValidationError
import requests
from .__version__ import VERSION

import functools
import sys

if sys.version_info > (3,):
    long = int
    unicode = str


# this decorator lets me use methods as both static and instance methods
class omnimethod(object):
    def __init__(self, func):
        self.func = func

    def __get__(self, instance, owner):
        return functools.partial(self.func, instance)


class PVWatts():
    '''
    A Python wrapper for NREL PVWatts V5.0.0 API
    '''

    PVWATTS_QUERY_URL = 'https://developer.nrel.gov/api/pvwatts/v5.json'
    api_key = 'DEMO_KEY'

    def __init__(self, api_key='DEMO_KEY', proxies=None):
        PVWatts.api_key = api_key
        self.proxies = proxies

    @omnimethod
    def validate_system_capacity(self, system_capacity):
        if system_capacity is None:
            return

        if not isinstance(system_capacity, (int, long, float)):
            raise PVWattsValidationError(
                'system_capacity must be int, long or float')

        if not (0.05 <= system_capacity and system_capacity <= 500000):
            raise PVWattsValidationError(
                'system_capacity must be >= 0.05 and <= 500000')

        return system_capacity

    @omnimethod
    def validate_module_type(self, module_type):
        if module_type is None:
            return

        if not isinstance(module_type, (int)):
            raise PVWattsValidationError(
                'module_type must be int, long or float')

        if module_type not in (0, 1, 2):
            raise PVWattsValidationError(
                'module_type must be 0, 1 or 2')

        return module_type

    @omnimethod
    def validate_losses(self, losses):
        if losses is None:
            return

        if not isinstance(losses, (int, long, float)):
            raise PVWattsValidationError('losses must be int, long or float')

        if not (-5 <= losses and losses <= 99):
            raise PVWattsValidationError('losses must be >= -5\% and <= 99%')

        return losses

    @omnimethod
    def validate_array_type(self, array_type):
        if array_type is None:
            return

        if not isinstance(array_type, (int)):
            raise PVWattsValidationError(
                'array_type must be int, long or float')

        if array_type not in (0, 1, 2, 3, 4):
            raise PVWattsValidationError(
                'array_type must be 0, 1, 2, 3 or 4')

        return array_type

    @omnimethod
    def validate_tilt(self, tilt):
        if tilt is None:
            return

        if not isinstance(tilt, (int, long, float)):
            raise PVWattsValidationError('tilt must be int, long or float')

        if not (0 <= tilt and tilt <= 90):
            raise PVWattsValidationError('tilt must be >= 0 and <= 90')

        return tilt

    @omnimethod
    def validate_azimuth(self, azimuth):
        if azimuth is None:
            return

        if not isinstance(azimuth, (int, long, float)):
            raise PVWattsValidationError('azimuth must be int, long or float')

        if not (0 <= azimuth and azimuth <= 360):
            raise PVWattsValidationError('azimuth must be >= 0 and <= 360')

        return azimuth

    @omnimethod
    def validate_lat(self, lat):
        if lat is None:
            return

        if not isinstance(lat, (int, long, float)):
            raise PVWattsValidationError('lat must be int, long or float')

        if not (-90 <= lat and lat <= 90):
            raise PVWattsValidationError('lat must be >= -90 and <= 90')

        return lat

    @omnimethod
    def validate_lon(self, lon):
        if lon is None:
            return

        if not isinstance(lon, (int, long, float)):
            raise PVWattsValidationError('lon must be int, long or float')

        if not (-180 <= lon and lon <= 180):
            raise PVWattsValidationError('lon must be >= -180 and <= 180')

        return lon

    @omnimethod
    def validate_dataset(self, dataset):
        if dataset is None:
            return

        if not isinstance(dataset, (str, unicode)):
            raise PVWattsValidationError('dataset must be str or unicode')

        if dataset not in ('tmy2', 'tmy3', 'intl'):
            raise PVWattsValidationError(
                'dataset must be \'tmy2\', \'tmy3\' or \'intl\'')

        return dataset

    @omnimethod
    def validate_radius(self, radius):
        if radius is None:
            return

        if not isinstance(radius, (int, long, float)):
            raise PVWattsValidationError('radius must be int, long or float')

        if not (0 <= radius):
            raise PVWattsValidationError('radius must be >= 0')

        return radius

    @omnimethod
    def validate_timeframe(self, timeframe):
        if timeframe is None:
            return

        if not isinstance(timeframe, (str, unicode)):
            raise PVWattsValidationError(
                'timeframe must be str or unicode')

        if timeframe not in ('hourly', 'monthly'):
            raise PVWattsValidationError(
                'dataset must be \'hourly\' or \'monthly\'')

        return timeframe

    @omnimethod
    def validate_dc_ac_ratio(self, dc_ac_ratio):
        if dc_ac_ratio is None:
            return

        if not isinstance(dc_ac_ratio, (int, long, float)):
            raise PVWattsValidationError(
                'dc_ac_ratio must be int, long or float')

        if not (0 < dc_ac_ratio):
            raise PVWattsValidationError(
                'dc_ac_ratio must be positive')

        return dc_ac_ratio

    @omnimethod
    def validate_gcr(self, gcr):
        if gcr is None:
            return

        if not isinstance(gcr, (int, long, float)):
            raise PVWattsValidationError('gcr must be int, long or float')

        if not (0 <= gcr and gcr <= 3):
            raise PVWattsValidationError('gcr must be >= 0 and <= 3')

        return gcr

    @omnimethod
    def validate_inv_eff(self, inv_eff):
        if inv_eff is None:
            return

        if not isinstance(inv_eff, (int, long, float)):
            raise PVWattsValidationError('inv_eff must be int, long or float')

        if not (90 <= inv_eff and inv_eff <= 99.5):
            raise PVWattsValidationError('inv_eff must be >= 90 and <= 99.5')

        return inv_eff

    @property
    def version(self):
        return VERSION

    @omnimethod
    def get_data(self, params={}):
        """
        Make the request and return the deserialided JSON from the response

        :param params: Dictionary mapping (string) query parameters to values
        :type params: dict
        :return: JSON object with the data fetched from that URL as a
                 JSON-format object.
        :rtype: (dict or array)

        """
        if self and hasattr(self, 'proxies') and self.proxies is not None:
            response = requests.request('GET',
                                        url=PVWatts.PVWATTS_QUERY_URL,
                                        params=params,
                                        headers={'User-Agent': ''.join(
                                                 ['pypvwatts/', VERSION,
                                                  ' (Python)'])},
                                        proxies=self.proxies)
        else:
            response = requests.request('GET',
                                        url=PVWatts.PVWATTS_QUERY_URL,
                                        params=params,
                                        headers={'User-Agent': ''.join(
                                                 ['pypvwatts/', VERSION,
                                                  ' (Python)'])})

        if response.status_code == 403:
            raise PVWattsError("Forbidden, 403")
        return response.json()

    @omnimethod
    def request(self, format=None, system_capacity=None, module_type=0,
                losses=12, array_type=1, tilt=None, azimuth=None,
                address=None, lat=None, lon=None, file_id=None, dataset='tmy3',
                radius=0, timeframe='monthly', dc_ac_ratio=None, gcr=None,
                inv_eff=None, callback=None):

        params = {
            'format': format,
            'system_capacity':
            PVWatts.validate_system_capacity(system_capacity),
            'module_type': PVWatts.validate_module_type(module_type),
            'losses': PVWatts.validate_losses(losses),
            'array_type': PVWatts.validate_array_type(array_type),
            'tilt': PVWatts.validate_tilt(tilt),
            'azimuth': PVWatts.validate_azimuth(azimuth),
            'address': address,
            'lat': PVWatts.validate_lat(lat),
            'lon': PVWatts.validate_lon(lon),
            'file_id': file_id,
            'dataset': PVWatts.validate_dataset(dataset),
            'radius': PVWatts.validate_radius(radius),
            'timeframe': PVWatts.validate_timeframe(timeframe),
            'dc_ac_ratio': PVWatts.validate_dc_ac_ratio(dc_ac_ratio),
            'gcr': PVWatts.validate_gcr(gcr),
            'inv_eff': PVWatts.validate_inv_eff(inv_eff),
            'callback': callback
        }

        params['api_key'] = PVWatts.api_key

        if self is not None:
            return PVWattsResult(self.get_data(params=params))
        return PVWattsResult(PVWatts.get_data(params=params))



# coding: utf-8
class PVWattsError(Exception):
    """
    Base class for PVWatts errors
    """
    def __init__(self, message):
        Exception.__init__(self, message)


class PVWattsValidationError(PVWattsError):
    """
    Validation error on request
    """
    def __init__(self, message):
        PVWattsError.__init__(self, message)



import os

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

from pypvwatts.__version__ import VERSION

setup(
    name='pypvwatts',
    version=VERSION,
    author='Miguel Paolino',
    author_email='mpaolino@gmail.com',
    url='https://github.com/mpaolino/pypvwatts',
    download_url='https://github.com/mpaolino/pypvwatts/archive/master.zip',
    description='Python wrapper for NREL PVWatts\'s API.',
    long_description=open('README.md').read(),
    packages=['pypvwatts'],
    provides=['pypvwatts'],
    requires=['requests'],
    install_requires=['requests >= 2.1.0'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'License :: OSI Approved :: MIT License',
        'Topic :: Internet',
        'Topic :: Internet :: WWW/HTTP',
    ],
    keywords='nrel pvwatts pypvwatts',
    license='MIT',
)



# coding: utf-8

class PVWattsResult(object):
    """
    Result class for PVWatts request
    """
    # Result's output fields to be accesed as properties of PVWattsResult
    shortcut_fields = ('poa_monthly', 'dc_monthly', 'ac_annual',
                       'solrad_annual', 'solrad_monthly', 'ac_monthly',
                       'ac', 'poa', 'dn', 'dc', 'df', 'tamb', 'tcell', 'wspd')

    def __init__(self, result):
        """
        Creates instance of PVWattsResult from the JSON API response
        """
        self.result = result

    @property
    def raw(self):
        return self.result

    def __getattr__(self, name):
        """
        Access outputs results as properties
        """
        result = None
        if name in PVWattsResult.shortcut_fields and 'outputs' in self.result:
            return self.result['outputs'][name]
        if name is not None:
            return self.result[name]
        return result

    def __unicode__(self):
        return unicode(self.result)

    def __str__(self):
        return self.__unicode__().encode('utf8')
